@@ -1,7 +1,7 @@
 package App::CSVUtils;
 
 our $DATE = '2016-11-16'; # DATE
-our $VERSION = '0.008'; # VERSION
+our $VERSION = '0.009'; # VERSION
 
 use 5.010001;
 use strict;
@@ -115,6 +115,18 @@ my %arg_filename_0 = (
     },
 );
 
+my %arg_filenames = (
+    filenames => {
+        'x.name.is_plural' => 1,
+        summary => 'Input CSV files',
+        schema => ['array*', of=>'filename*'],
+        req => 1,
+        pos => 0,
+        greedy => 1,
+        cmdline_aliases => {f=>{}},
+    },
+);
+
 my %arg_field_1 = (
     field => {
         summary => 'Field name',
@@ -185,6 +197,7 @@ my %arg_with_data_rows = (
 $SPEC{csvutil} = {
     v => 1.1,
     summary => 'Perform action on a CSV file',
+    'x.no_index' => 1,
     args => {
         action => {
             schema => ['str*', in=>[
@@ -318,8 +331,36 @@ sub csvutil {
             $res .= _get_csv_row($csv, $row, $i);
         } elsif ($action eq 'add-field') {
             if ($i == 1) {
-                $field_idx = @$row;
-                push @$row, $args{field};
+                if (defined $args{_at}) {
+                    $field_idx = $args{_at}-1;
+                } elsif (defined $args{before}) {
+                    for (0..$#{$row}) {
+                        if ($row->[$_] eq $args{before}) {
+                            $field_idx = $_;
+                            last;
+                        }
+                    }
+                    return [400, "Field '$args{before}' not found"]
+                        unless defined $field_idx;
+                } elsif (defined $args{after}) {
+                    for (0..$#{$row}) {
+                        if ($row->[$_] eq $args{after}) {
+                            $field_idx = $_+1;
+                            last;
+                        }
+                    }
+                    return [400, "Field '$args{after}' not found"]
+                        unless defined $field_idx;
+                } else {
+                    $field_idx = @$row;
+                }
+                splice @$row, $field_idx, 0, $args{field};
+                for (keys %field_idxs) {
+                    if ($field_idxs{$_} >= $field_idx) {
+                        $field_idxs{$_}++;
+                    }
+                }
+                $fields = $row;
             } else {
                 unless ($code) {
                     $code = _compile($args{eval});
@@ -330,14 +371,14 @@ sub csvutil {
                         return [412, "Field '$args{field}' already exists"];
                     }
                 }
-                if (!defined($row->[$field_idx])) {
+                {
                     local $_;
                     local $main::row = $row;
                     local $main::rownum = $i;
                     eval { $_ = $code->() };
                     die "Error while adding field '$args{field}' for row #$i: $@\n"
                         if $@;
-                    $row->[$field_idx] = $_;
+                    splice @$row, $field_idx, 0, $_;
                 }
             }
             $res .= _get_csv_row($csv, $row, $i);
@@ -426,18 +467,43 @@ $SPEC{csv_add_field} = {
 Your Perl code (-e) will be called for each row (excluding the header row) and
 should return the value for the new field. `$main::row` is available and
 contains the current row, while `$main::rownum` contains the row number (2 means
-the first data row). Field will be added as the last field.
+the first data row).
+
+Field by default will be added as the last field, unless you specify one of
+`--after` (to put after a certain field), `--before` (to put before a certain
+field), or `--at` (to put at specific position, 1 means as the first field).
 
 _
     args => {
         %arg_filename_0,
         %arg_field_1_nocomp,
         %arg_eval_2,
+        after => {
+            summary => 'Put the new field after specified field',
+            schema => 'str*',
+        },
+        before => {
+            summary => 'Put the new field before specified field',
+            schema => 'str*',
+        },
+        at => {
+            summary => 'Put the new field at specific position '.
+                '(1 means as first field)',
+            schema => ['int*', min=>1],
+        },
+    },
+    args_rels => {
+        choose_one => [qw/after before at/],
     },
 };
 sub csv_add_field {
     my %args = @_;
-    csvutil(%args, action=>'add-field');
+    csvutil(
+        %args, action=>'add-field',
+        _after  => $args{after},
+        _before => $args{before},
+        _at     => $args{at},
+    );
 }
 
 $SPEC{csv_list_field_names} = {
@@ -627,6 +693,101 @@ sub csv_convert_to_hash {
             _row_number=>$args{row_number} // 2);
 }
 
+$SPEC{csv_concat} = {
+    v => 1.1,
+    summary => 'Concatenate several CSV files together, '.
+        'collecting all the fields',
+    description => <<'_',
+
+Example, concatenating this CSV:
+
+    col1,col2
+    1,2
+    3,4
+
+and:
+
+    col2,col4
+    a,b
+    c,d
+    e,f
+
+and:
+
+    col3
+    X
+    Y
+
+will result in:
+
+    col1,col2,col4,col3
+    1,2,
+    3,4,
+    ,a,b
+    ,c,d
+    ,e,f
+    ,,,X
+    ,,,Y
+
+_
+    args => {
+        %arg_filenames,
+    },
+};
+sub csv_concat {
+    require Text::CSV_XS;
+
+    my %args = @_;
+
+    my %res_field_idxs;
+    my @rows;
+
+    for my $filename (@{ $args{filenames} }) {
+        my $csv = Text::CSV_XS->new({binary => 1});
+        open my($fh), "<:encoding(utf8)", $filename or
+            return [500, "Can't open input filename '$filename': $!"];
+        my $i = 0;
+        my $fields;
+        while (my $row = $csv->getline($fh)) {
+            $i++;
+            if ($i == 1) {
+                $fields = $row;
+                for my $field (@$fields) {
+                    unless (exists $res_field_idxs{$field}) {
+                        $res_field_idxs{$field} = keys(%res_field_idxs);
+                    }
+                }
+                next;
+            }
+            my $res_row = [];
+            for my $j (0..$#{$row}) {
+                my $field = $fields->[$j];
+                $res_row->[ $res_field_idxs{$field} ] = $row->[$j];
+            }
+            push @rows, $res_row;
+        }
+    } # for each filename
+
+    my $num_fields = keys %res_field_idxs;
+    my $res = "";
+    my $csv = Text::CSV_XS->new({binary => 1});
+
+    # generate header
+    my $status = $csv->combine(
+        sort { $res_field_idxs{$a} <=> $res_field_idxs{$b} }
+            keys %res_field_idxs)
+        or die "Error in generating result header row: ".$csv->error_input;
+    $res .= $csv->string . "\n";
+    for my $i (0..$#rows) {
+        my $row = $rows[$i];
+        $row->[$num_fields-1] = undef if @$row < $num_fields;
+        my $status = $csv->combine(@$row)
+            or die "Error in generating data row #".($i+1).": ".
+            $csv->error_input;
+        $res .= $csv->string . "\n";
+    }
+    [200, "OK", $res, {"cmdline.skip_format"=>1}];
+}
 
 1;
 # ABSTRACT: CLI utilities related to CSV
@@ -643,7 +804,7 @@ App::CSVUtils - CLI utilities related to CSV
 
 =head1 VERSION
 
-This document describes version 0.008 of App::CSVUtils (from Perl distribution App-CSVUtils), released on 2016-11-16.
+This document describes version 0.009 of App::CSVUtils (from Perl distribution App-CSVUtils), released on 2016-11-16.
 
 =head1 DESCRIPTION
 
@@ -655,6 +816,8 @@ This distribution contains the following CLI utilities:
 
 =item * L<csv-avg>
 
+=item * L<csv-concat>
+
 =item * L<csv-convert-to-hash>
 
 =item * L<csv-delete-field>
@@ -683,7 +846,11 @@ Add a field to CSV file.
 Your Perl code (-e) will be called for each row (excluding the header row) and
 should return the value for the new field. C<$main::row> is available and
 contains the current row, while C<$main::rownum> contains the row number (2 means
-the first data row). Field will be added as the last field.
+the first data row).
+
+Field by default will be added as the last field, unless you specify one of
+C<--after> (to put after a certain field), C<--before> (to put before a certain
+field), or C<--at> (to put at specific position, 1 means as the first field).
 
 This function is not exported.
 
@@ -691,6 +858,18 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<after> => I<str>
+
+Put the new field after specified field.
+
+=item * B<at> => I<int>
+
+Put the new field at specific position (1 means as first field).
+
+=item * B<before> => I<str>
+
+Put the new field before specified field.
+
 =item * B<eval>* => I<str>
 
 Perl code to do munging.
@@ -749,6 +928,64 @@ that contains extra information.
 Return value:  (any)
 
 
+=head2 csv_concat(%args) -> [status, msg, result, meta]
+
+Concatenate several CSV files together, collecting all the fields.
+
+Example, concatenating this CSV:
+
+ col1,col2
+ 1,2
+ 3,4
+
+and:
+
+ col2,col4
+ a,b
+ c,d
+ e,f
+
+and:
+
+ col3
+ X
+ Y
+
+will result in:
+
+ col1,col2,col4,col3
+ 1,2,
+ 3,4,
+ ,a,b
+ ,c,d
+ ,e,f
+ ,,,X
+ ,,,Y
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<filenames>* => I<array[filename]>
+
+Input CSV files.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 csv_convert_to_hash(%args) -> [status, msg, result, meta]
 
 Return a hash of field names as keys and first row as values.
@@ -1017,43 +1254,7 @@ that contains extra information.
 
 Return value:  (any)
 
-
-=head2 csvutil(%args) -> [status, msg, result, meta]
-
-Perform action on a CSV file.
-
-This function is not exported.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<action>* => I<str>
-
-=item * B<eval> => I<str>
-
-Perl code to do munging.
-
-=item * B<field> => I<str>
-
-Field name.
-
-=item * B<filename>* => I<filename>
-
-Input CSV file.
-
-=back
-
-Returns an enveloped result (an array).
-
-First element (status) is an integer containing HTTP status code
-(200 means OK, 4xx caller error, 5xx function error). Second element
-(msg) is a string containing error message, or 'OK' if status is
-200. Third element (result) is optional, the actual result. Fourth
-element (meta) is called result metadata and is optional, a hash
-that contains extra information.
-
-Return value:  (any)
+=for Pod::Coverage ^(csvutil)$
 
 =head1 HOMEPAGE
 
